# a quiz app that tooks the question from http://www.jservice.io/ and put it into the db with https://github.com/nuqui-chatbot/nuqui-question-database
from .dbobjects import User, Score, Question, Meal
from . import SESSION, QUESTIONS
from random import randint, shuffle
import datetime


def create_user(user_id, user_name):
    session = SESSION()
    user_score = Score(points=0, latest_points=0)
    user = User(id=user_id, name=user_name, score=user_score)
    session.add(user_score)
    session.add(user)
    session.commit()
    session.close()


def remove_user(user_id):
    session = SESSION()
    user = session.query(User).filter_by(id=user_id).one()
    session.delete(user)
    session.commit()
    session.close()


def get_predefined_question_dict_with_random_answers(user_id):
    # get the questions the user already answered and delete them from the list of possible questions
    session = SESSION()
    user = session.query(User).filter_by(id=user_id).one()
    questions_id = user.questions
    possible_questions = [question for question in QUESTIONS if question not in questions_id]
    # select random question from list of possible questions
    question = possible_questions[randint(0, len(possible_questions))]
    # get three random answers and the right answer and shuffle them
    possible_answers = _get_three_random_answers(question)
    possible_answers.append(question.answer)
    shuffle(possible_answers)
    # create question dict and add the possible answers, then return the dict
    question_dict = question.to_dictionary()
    question_dict['answer'] = possible_answers
    user.questions.append(question)
    session.commit()
    session.close()
    return question_dict


def _get_three_random_answers(ori_question):
    random_answers_answer = [question.answer for question in QUESTIONS if question != ori_question]
    return [random_answers_answer[randint(0, len(random_answers_answer))] for x in range(0, 3)]


def evaluate(answer, question_id):
    session = SESSION()
    question = session.query(Question).filter_by(id=question_id).one()
    session.close()
    return question.answer == answer


# ToDo: Calculate new score and return a dictionary containing succes, right answer and achieved points


# food as dict eg:
# {
#    "apple" : {
#        "amount": 1,
#        "calories": 50
#    }
# }
def add_meal(user_id, food_dict):
    session = SESSION()
    transformed_dict = _transform_food_dict_to_string_and_cals(food_dict)
    meal = Meal(timestamp=datetime.datetime.now(), calories=transformed_dict["calories"],
                food=transformed_dict["food_string"])
    session.add(meal)
    user = session.query(User).filter_by(id=user_id).one()
    user.meals.append(meal)
    session.commit()
    session.close()


def _transform_food_dict_to_string_and_cals(food_dict):
    result_list = []
    calories = 0
    for key, value in food_dict.items():
        calories += value["calories"]
        if not result_list:
            result_list.append(str(value["amount"]) + " " + key)
        else:
            result_list.append("," + str(value["amount"]) + " " + key)

    return {
        "calories": calories,
        "food_string": ''.join(result_list)
    }



from sqlalchemy import Column, Integer, String, Table, ForeignKey, DateTime
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship

#declare a mapping
global Base
Base = declarative_base()

User_question = Table(
        'user_question', Base.metadata,
        Column('user_id', Integer, ForeignKey('user.id')),
        Column('question_id', Integer, ForeignKey('downloaded_question.id'))
 )


User_meal = Table(
    'user_meal', Base.metadata,
    Column('user_id', Integer, ForeignKey('user.id')),
    Column('meal_id', Integer, ForeignKey('meal.id'))
)


class Question(Base):
    __tablename__ = 'downloaded_question'

    id = Column(Integer, primary_key=True)
    answer = Column(String)
    question = Column(String)
    value = Column(Integer)

    def to_dictionary(self):
        return {
                "id": self.id,
                "question": self.question,
                "answer": self.answer,
                "value": self.value
                }


class User(Base):
    __tablename__ = 'user'

    id = Column(Integer, primary_key=True)
    name = Column(String)
    score = relationship("Score", uselist=False, cascade="delete")
    questions = relationship("Question", secondary=User_question, cascade="delete")
    meals = relationship("Meal", secondary=User_meal, cascade="delete")

    def to_dictionary(self):
        return {
            "id": self.id,
            "name": self.name,
            "points": self.score.points,
            "last_points": self.score.latest_points,
            "questions": self.questions
        }


class Score(Base):
    __tablename__ = 'score'

    id = Column(Integer, primary_key=True)
    points = Column(Integer)
    latest_points = Column(Integer)
    user_id = Column(Integer, ForeignKey("user.id"))


class Meal(Base):
    __tablename__='meal'

    id = Column(Integer, primary_key=True)
    timestamp = Column(DateTime)
    calories = Column(Integer)
    food = Column(String)

    def to_dictionary(self):
        return {
            "id": self.id,
            "timestamp": self.timestamp,
            "calories": self.calories,
            "food": self.food
        }

